@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "07/02/2020"
+__date__ = "11/12/2020"
 
 import numpy
 
@@ -67,7 +67,7 @@ class DisplayComponentsWidget(qt.QMainWindow):
 
         self._toolbar = qt.QToolBar(parent=self)
         self._toolbar.setIconSize(self._toolbar.iconSize() * 1.2)
-        if not silx.resources.is_dir("darfix"):
+        if "darfix" not in silx.resources._RESOURCE_DIRECTORIES:
             silx.resources.register_resource_directory("darfix", "darfix.resources")
         curves_icon = icons.getQIcon('darfix:gui/icons/curves')
         scatter_icon = icons.getQIcon('darfix:gui/icons/scatter')
@@ -109,7 +109,7 @@ class DisplayComponentsWidget(qt.QMainWindow):
     def showButton(self):
         self._computeB.show()
 
-    def setComponents(self, components, W, dimensions):
+    def setComponents(self, components, W, dimensions, shape):
         """
         Components setter. Updates the plots with the components and their
         corresponding rocking curves.
@@ -121,6 +121,7 @@ class DisplayComponentsWidget(qt.QMainWindow):
         self.components = components
         self.W = W
         self.dimensions = dimensions
+        self.dimensions_shape = shape
         self._activateCurvesPlot(True)
         self._sv_components.setStack(self.components)
         self._plot_rocking_curves.getPlot().clear()
@@ -214,14 +215,13 @@ class DisplayComponentsWidget(qt.QMainWindow):
         elif axis != -1 and axis is not None:
             text = self._plot_rocking_curves.getDimensionsCB().currentText()
             self._plot_rocking_curves.getPlot().setGraphYLabel(text)
-            self.dimension = axis - 1
+            self.dimension = len(self.dimensions) - axis
             self._plot_rocking_curves.clear()
             self._plot_rocking_curves.getBrowser().setEnabled(True)
-            self._plot_rocking_curves.getBrowser().setRange(0, len(self.dimensions[text]) - 1)
+            self._plot_rocking_curves.getBrowser().setRange(0, self.dimensions_shape[self.dimension] - 1)
             self._plot_rocking_curves.getBrowser().setValue(0)
             for i in range(len(self.W.T)):
-                dimensions_shape = numpy.array(self.dimensions.values()).shape
-                W = numpy.take(self.W.T[i].reshape(dimensions_shape), self.index, self.dimension)
+                W = numpy.take(self.W.T[i].reshape(self.dimensions_shape), self.index, self.dimension)
                 self._plot_rocking_curves.addCurve(numpy.arange(len(W)), W, legend=str(i))
         else:
             raise ValueError('Axis %s is not managed' % axis)
@@ -234,10 +234,26 @@ class DisplayComponentsWidget(qt.QMainWindow):
         self.index = index
         self._plot_rocking_curves.clear()
         for i in range(len(self.W.T)):
-            dimensions_shape = numpy.array(self.dimensions.values()).shape
-            W = numpy.take(self.W.T[i].reshape(dimensions_shape), self.index, self.dimension)
+            W = numpy.take(self.W.T[i].reshape(self.dimensions_shape), self.index, self.dimension)
             self._plot_rocking_curves.addCurve(numpy.arange(len(W)), W, legend=str(i))
 
+    def setStackViewColormap(self, colormap):
+        """
+        Sets the stackView colormap
+
+        :param colormap: Colormap to set
+        :type colormap: silx.gui.colors.Colormap
+        """
+        self._sv_components.setColormap(colormap)
+
+    def getStackViewColormap(self):
+        """
+        Returns the colormap from the stackView
+
+        :rtype: silx.gui.colors.Colormap
+        """
+        return self._sv_components.getColormap()
+
 
 class PlotRockingCurves(qt.QMainWindow):
     """
