@@ -26,7 +26,9 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "07/04/2020"
+__date__ = "11/12/2020"
+
+import logging
 
 from silx.gui import qt
 from silx.gui.colors import Colormap
@@ -38,6 +40,8 @@ from darfix.core.imageOperations import Method
 
 from .operationThread import OperationThread
 
+_logger = logging.getLogger(__file__)
+
 
 class _ParametersDock(qt.QDockWidget):
 
@@ -65,6 +69,17 @@ class _ParametersDock(qt.QDockWidget):
         self.abortBS.hide()
         methodLabel = qt.QLabel("Method:")
         bgLabel = qt.QLabel("Background:")
+        methodLabel.setMargin(0)
+        # Step widget
+        self.stepWidget = qt.QWidget()
+        stepLayout = qt.QHBoxLayout()
+        stepLayout.addWidget(qt.QLabel("Step:"))
+        self.step = qt.QLineEdit("1")
+        self.step.setSizePolicy(qt.QSizePolicy.Ignored, qt.QSizePolicy.Preferred)
+        stepLayout.setContentsMargins(0, 0, 0, 0)
+        stepLayout.addWidget(self.step)
+        self.stepWidget.setLayout(stepLayout)
+        # Chunks widget
         self.chunksWidget = qt.QWidget()
         chunksLayout = qt.QHBoxLayout()
         chunksLabel = qt.QLabel("Chunks:")
@@ -75,14 +90,30 @@ class _ParametersDock(qt.QDockWidget):
         chunksLayout.addWidget(chunksLabel)
         chunksLayout.addWidget(self.verticalChunkSize)
         chunksLayout.addWidget(self.horizontalChunkSize)
-        chunksLayout.setContentsMargins(0, 0, 1, 1)
+        chunksLayout.setContentsMargins(0, 0, 0, 0)
         self.chunksWidget.setLayout(chunksLayout)
-        self.layout.addWidget(methodLabel, 1, 0, 1, 0.7)
-        self.layout.addWidget(bgLabel, 2, 0, 1, 0.7)
+
+        self.inDiskCheckbox = qt.QCheckBox("Use chunks to compute median")
+        self.inDiskWidget = qt.QWidget()
+        inDiskLayout = qt.QVBoxLayout()
+        inDiskLayout.addWidget(self.stepWidget)
+        inDiskLayout.addWidget(self.chunksWidget)
+        inDiskLayout.addWidget(self.inDiskCheckbox, alignment=qt.Qt.AlignRight)
+        self.chunksWidget.hide()
+        self.inDiskCheckbox.stateChanged.connect(self._toggleChunks)
+        inDiskLayout.setContentsMargins(0, 0, 0, 0)
+        self.inDiskWidget.setLayout(inDiskLayout)
+
+        self.layout.addWidget(methodLabel, 1, 0, 1, 1)
+        self.layout.addWidget(bgLabel, 2, 0, 1, 1)
         self.layout.addWidget(self.bsMethodsCB, 1, 1, 1, 1)
         self.layout.addWidget(self.bsBackgroundCB, 2, 1, 1, 1)
-        self.layout.addWidget(self.chunksWidget, 3, 0, 1, 2)
-        self.chunksWidget.hide()
+        self.layout.addWidget(self.inDiskWidget, 3, 0, 1, 2)
+        self.layout.addWidget(self.computeBS, 4, 1, 1, 1)
+        self.layout.addWidget(self.abortBS, 4, 0, 1, 1)
+        self.computeBS.hide()
+        self.abortBS.hide()
+        self.inDiskWidget.hide()
         # Hot pixel removal
         hpLabel = qt.QLabel("Hot Pixel Removal")
         hpLabel.setFont(font)
@@ -95,13 +126,37 @@ class _ParametersDock(qt.QDockWidget):
         self.computeHP = qt.QPushButton("Compute")
         self.abortHP = qt.QPushButton("Abort")
         self.layout.addWidget(self.hpSizeCB, 1, 3, 1, 1)
-        self.layout.addWidget(self.computeHP, 3, 3, 1, 1)
-        self.layout.addWidget(self.abortHP, 3, 2, 1, 1)
+        self.layout.addWidget(self.computeHP, 4, 3, 1, 1)
+        self.layout.addWidget(self.abortHP, 4, 2, 1, 1)
+        self.computeHP.hide()
         self.abortHP.hide()
+        # Threshold removal
+        tpLabel = qt.QLabel("Threshold Removal")
+        tpLabel.setFont(font)
+        self.layout.addWidget(tpLabel, 0, 4, 1, 2)
+        bottomLabel = qt.QLabel("Bottom threshold:")
+        self.layout.addWidget(bottomLabel, 1, 4, 1, 1)
+        self.bottomLE = qt.QLineEdit("0")
+        self.bottomLE.setValidator(qt.QIntValidator())
+        self.bottomLE.setSizePolicy(qt.QSizePolicy.Ignored, qt.QSizePolicy.Preferred)
+        self.computeTP = qt.QPushButton("Compute")
+        self.abortTP = qt.QPushButton("Abort")
+        self.layout.addWidget(self.bottomLE, 1, 5, 1, 1)
+        self.layout.addWidget(self.computeTP, 4, 5, 1, 1)
+        self.layout.addWidget(self.abortTP, 4, 4, 1, 1)
+        self.computeTP.hide()
+        self.abortTP.hide()
 
+        self.layout.setHorizontalSpacing(10)
         widget.setLayout(self.layout)
         self.setWidget(widget)
 
+    def _toggleChunks(self, checked):
+        widgets = [self.chunksWidget, self.stepWidget]
+
+        widgets[1 if checked else 0].hide()
+        widgets[0 if checked else 1].show()
+
 
 class NoiseRemovalDialog(qt.QDialog):
     """
@@ -114,17 +169,28 @@ class NoiseRemovalDialog(qt.QDialog):
         qt.QDialog.__init__(self, parent)
         self.setWindowFlags(qt.Qt.Widget)
         types = qt.QDialogButtonBox.Ok
-        _buttons = qt.QDialogButtonBox(parent=self)
-        _buttons.setStandardButtons(types)
-        resetB = _buttons.addButton(_buttons.Reset)
+        self._buttons = qt.QDialogButtonBox(parent=self)
+        self._buttons.setStandardButtons(types)
+        self._buttons.setEnabled(False)
+        resetB = self._buttons.addButton(self._buttons.Reset)
         self.mainWindow = NoiseRemovalWidget(parent=self)
         self.mainWindow.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.setLayout(qt.QVBoxLayout())
         self.layout().addWidget(self.mainWindow)
-        self.layout().addWidget(_buttons)
+        self.layout().addWidget(self._buttons)
 
-        _buttons.accepted.connect(self.okSignal.emit)
+        self._buttons.accepted.connect(self.okSignal.emit)
         resetB.clicked.connect(self.mainWindow.resetStack)
+        self.mainWindow.computingSignal.connect(self._toggleButton)
+
+    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+
+        if dataset is not None:
+            self._buttons.setEnabled(True)
+            self.mainWindow.setDataset(dataset, indices, bg_indices, bg_dataset)
+
+    def _toggleButton(self, state):
+        self._buttons.button(qt.QDialogButtonBox.Ok).setEnabled(not state)
 
 
 class NoiseRemovalWidget(qt.QMainWindow):
@@ -136,6 +202,8 @@ class NoiseRemovalWidget(qt.QMainWindow):
     frames, an image is computed either using the mean or the median.
     """
 
+    computingSignal = qt.Signal(bool)
+
     def __init__(self, parent=None):
         qt.QMainWindow.__init__(self, parent)
 
@@ -150,75 +218,23 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._sv = StackViewMainWindow()
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
                                       normalization=darfix.config.DEFAULT_COLORMAP_NORM))
+
+        self._size = self._parametersDock.hpSizeCB.currentText()
+        self._method = self._parametersDock.bsMethodsCB.currentText()
+        self._background = self._parametersDock.bsBackgroundCB.currentText()
+        self._bottom_threshold = self._parametersDock.bottomLE.text()
         self.setCentralWidget(self._sv)
         self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._parametersDock)
+
+        # Add connections
         self._parametersDock.computeBS.clicked.connect(self.__computeBS)
         self._parametersDock.computeHP.clicked.connect(self.__computeHP)
+        self._parametersDock.computeTP.clicked.connect(self.__computeTP)
         self._parametersDock.abortBS.clicked.connect(self.__abortBS)
         self._parametersDock.abortHP.clicked.connect(self.__abortHP)
-
-    def __computeBS(self):
-        """
-        Function that starts the thread to compute the background
-        subtraction.
-        """
-        method = self._parametersDock.bsMethodsCB.currentText()
-        background = self._parametersDock.bsBackgroundCB.currentText()
-        self._thread = OperationThread(self, self._update_dataset.apply_background_subtraction)
-        bg = None
-        chunks = None
-        if background == "Dark data":
-            bg = self.bg_dataset
-        elif background == "Low intensity data":
-            bg = self.bg_indices
-        if not self._update_dataset.in_memory:
-            chunks = [int(self._parametersDock.verticalChunkSize.text()),
-                      int(self._parametersDock.horizontalChunkSize.text())]
-        self._thread.setArgs(bg, method, self.indices, chunks)
-        self._thread.finished.connect(self._updateData)
-        self._parametersDock.abortBS.show()
-        self._thread.start()
-        self._parametersDock.computeBS.setEnabled(False)
-        self._parametersDock.computeHP.setEnabled(False)
-
-    def __computeHP(self):
-        """
-        Function that starts the thread to compute the hot pixel
-        removal.
-        """
-        size = self._parametersDock.hpSizeCB.currentText()
-        self._thread = OperationThread(self, self._update_dataset.apply_hot_pixel_removal)
-        self._thread.setArgs(int(size), self.indices)
-        self._thread.finished.connect(self._updateData)
-        self._parametersDock.abortHP.show()
-        self._thread.start()
-        self._parametersDock.computeBS.setEnabled(False)
-        self._parametersDock.computeHP.setEnabled(False)
-
-    def __abortBS(self):
-        self._parametersDock.abortBS.setEnabled(False)
-        self._update_dataset.stop_operation(Operation.BS)
-
-    def __abortHP(self):
-        self._parametersDock.abortHP.setEnabled(False)
-        self._update_dataset.stop_operation(Operation.HP)
-
-    def _updateData(self):
-        """
-        Updates the stack with the data computed in the thread
-        """
-        self._thread.finished.disconnect(self._updateData)
-        self._parametersDock.abortBS.setEnabled(True)
-        self._parametersDock.abortHP.setEnabled(True)
-        self._parametersDock.abortBS.hide()
-        self._parametersDock.abortHP.hide()
-        self._parametersDock.computeBS.setEnabled(True)
-        self._parametersDock.computeHP.setEnabled(True)
-        if self._thread.data is not None:
-            self._update_dataset = self._thread.data
-            self.setStack(self._update_dataset)
-        else:
-            print("\nComputation aborted")
+        self._parametersDock.abortTP.clicked.connect(self.__abortTP)
+        self._parametersDock.bsMethodsCB.currentTextChanged.connect(self._toggleMethod)
+        self._parametersDock.bsBackgroundCB.currentIndexChanged.connect(self._toggleInDiskWidget)
 
     def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
@@ -233,14 +249,10 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
-        if not dataset.in_memory:
-            self._parametersDock.chunksWidget.show()
-            self._parametersDock.layout.addWidget(self._parametersDock.abortBS, 4, 0, 1, 1)
-            self._parametersDock.layout.addWidget(self._parametersDock.computeBS, 4, 1, 1, 1)
-        else:
-            self._parametersDock.chunksWidget.hide()
-            self._parametersDock.layout.addWidget(self._parametersDock.abortBS, 3, 0, 1, 1)
-            self._parametersDock.layout.addWidget(self._parametersDock.computeBS, 3, 1, 1, 1)
+
+        self._parametersDock.computeBS.show()
+        self._parametersDock.computeHP.show()
+        self._parametersDock.computeTP.show()
 
         """
         Sets the available background for the user to choose.
@@ -250,9 +262,123 @@ class NoiseRemovalWidget(qt.QMainWindow):
             self._parametersDock.bsBackgroundCB.addItem("Dark data")
         if bg_indices is not None:  # TODO: modify
             self._parametersDock.bsBackgroundCB.addItem("Low intensity data")
-
         self._parametersDock.bsBackgroundCB.addItem("Data")
 
+    def __computeBS(self):
+        """
+        Function that starts the thread to compute the background
+        subtraction.
+        """
+        self._background = self._parametersDock.bsBackgroundCB.currentText()
+        if self._background == "Dark data":
+            bg = self.bg_dataset
+        elif self._background == "Low intensity data":
+            bg = self.bg_indices
+        else:
+            bg = None
+
+        chunks = None
+        step = None
+        if self._parametersDock.inDiskWidget.isVisible():
+            if self._parametersDock.inDiskCheckbox.isChecked():
+                chunks = [int(self._parametersDock.verticalChunkSize.text()),
+                          int(self._parametersDock.horizontalChunkSize.text())]
+            else:
+                step = int(self._parametersDock.step.text())
+
+        self._thread = OperationThread(self, self._update_dataset.apply_background_subtraction)
+        self._method = self._parametersDock.bsMethodsCB.currentText()
+        self._thread.setArgs(bg, self._method, self.indices, step, chunks)
+        self._thread.finished.connect(self._updateData)
+        self._parametersDock.abortBS.show()
+        self._thread.start()
+        self._setComputingState(True)
+
+    def __computeHP(self):
+        """
+        Function that starts the thread to compute the hot pixel
+        removal.
+        """
+        self._size = self._parametersDock.hpSizeCB.currentText()
+        self._thread = OperationThread(self, self._update_dataset.apply_hot_pixel_removal)
+        self._thread.setArgs(int(self._size), self.indices)
+        self._thread.finished.connect(self._updateData)
+        self._parametersDock.abortHP.show()
+        self._thread.start()
+        self._setComputingState(True)
+
+    def __computeTP(self):
+        """
+        Function that starts the thread to compute the threshold
+        removal.
+        """
+        self._bottom_threshold = self._parametersDock.bottomLE.text()
+        self._thread = OperationThread(self, self._update_dataset.apply_threshold_removal)
+        self._thread.setArgs(int(self._bottom_threshold))
+        self._thread.finished.connect(self._updateData)
+        self._parametersDock.abortTP.show()
+        self._thread.start()
+        self._setComputingState(True)
+
+    def __abortBS(self):
+        self._parametersDock.abortBS.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.BS)
+
+    def __abortHP(self):
+        self._parametersDock.abortHP.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.HP)
+
+    def __abortTP(self):
+        self._parametersDock.abortTP.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.TP)
+
+    def _setComputingState(self, state):
+
+        self._parametersDock.computeBS.setEnabled(not state)
+        self._parametersDock.computeHP.setEnabled(not state)
+        self._parametersDock.computeTP.setEnabled(not state)
+
+        self.computingSignal.emit(state)
+
+    def _updateData(self):
+        """
+        Updates the stack with the data computed in the thread
+        """
+        try:
+            self._thread.finished.disconnect(self._updateData)
+        except TypeError as e:
+            _logger.warning(e)
+        self._parametersDock.abortBS.setEnabled(True)
+        self._parametersDock.abortHP.setEnabled(True)
+        self._parametersDock.abortTP.setEnabled(True)
+        self._parametersDock.abortBS.hide()
+        self._parametersDock.abortHP.hide()
+        self._parametersDock.abortTP.hide()
+        self._setComputingState(False)
+        if self._thread.data is not None:
+            self._update_dataset = self._thread.data
+            self.setStack(self._update_dataset)
+        else:
+            print("\nComputation aborted")
+
+    def _toggleMethod(self, text):
+        if text == Method.mean.value:
+            self._parametersDock.inDiskWidget.hide()
+        elif text == Method.median.value:
+            self._toggleInDiskWidget(self._parametersDock.bsBackgroundCB.currentIndex())
+
+    def _toggleInDiskWidget(self, index):
+        if self._parametersDock.bsMethodsCB.currentText() == Method.median.value:
+            if self.bg_dataset is None:
+                self._parametersDock.inDiskWidget.hide() if self.dataset.in_memory else \
+                    self._parametersDock.inDiskWidget.show()
+            elif not (index or self.bg_dataset.in_memory) or (index and not self.dataset.in_memory):
+                self._parametersDock.inDiskWidget.show()
+            else:
+                self._parametersDock.inDiskWidget.hide()
+        else:
+            self._parametersDock.inDiskWidget.hide()
+
     def getDataset(self):
         return self._update_dataset, self.indices, self.bg_indices, self. bg_dataset
 
@@ -277,3 +403,57 @@ class NoiseRemovalWidget(qt.QMainWindow):
         nframe = self._sv.getFrameNumber()
         self._sv.setStack(dataset.get_data(self.indices))
         self._sv.setFrameNumber(nframe)
+
+    def getStackViewColormap(self):
+        """
+        Returns the colormap from the stackView
+
+        :rtype: silx.gui.colors.Colormap
+        """
+        return self._sv.getColormap()
+
+    def setStackViewColormap(self, colormap):
+        """
+        Sets the stackView colormap
+
+        :param colormap: Colormap to set
+        :type colormap: silx.gui.colors.Colormap
+        """
+        self._sv.setColormap(colormap)
+
+    @property
+    def size(self):
+        return self._size
+
+    @size.setter
+    def size(self, size):
+        self._size = size
+        self._parametersDock.hpSizeCB.setCurrentText(size)
+
+    @property
+    def method(self):
+        return self._method
+
+    @method.setter
+    def method(self, method):
+        self._method = method
+        self._parametersDock.bsMethodsCB.setCurrentText(method)
+
+    @property
+    def background(self):
+        return self._background
+
+    @background.setter
+    def background(self, background):
+        if self._parametersDock.bsBackgroundCB.findText(background) >= 0:
+            self._background = background
+            self._parametersDock.bsBackgroundCB.setCurrentText(background)
+
+    @property
+    def bottom_threshold(self):
+        return self._bottom_threshold
+
+    @bottom_threshold.setter
+    def bottom_threshold(self, bottom):
+        self._bottom_threshold = bottom
+        self._parametersDock.bottomLE.setText(bottom)
