@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "07/04/2020"
+__date__ = "11/12/2020"
 
 # import os
 
@@ -40,6 +40,8 @@ from silx.gui.plot.items.roi import RectangleROI
 from silx.gui.plot.tools.roi import RegionOfInterestManager, RegionOfInterestTableWidget
 
 import darfix
+from darfix.core.dataset import Operation
+from .operationThread import OperationThread
 
 
 _logger = logging.getLogger(__file__)
@@ -65,11 +67,14 @@ class ROISelectionWidget(qt.QWidget):
         _buttons = qt.QDialogButtonBox(parent=self)
         self._okB = _buttons.addButton(_buttons.Ok)
         self._applyB = _buttons.addButton(_buttons.Apply)
+        self._abortB = _buttons.addButton(_buttons.Abort)
         self._resetB = _buttons.addButton(_buttons.Reset)
+        self._abortB.hide()
 
         self._applyB.clicked.connect(self.applyRoi)
         self._okB.clicked.connect(self.apply)
         self._resetB.clicked.connect(self.resetStack)
+        self._abortB.clicked.connect(self.abort)
 
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
                                       normalization=darfix.config.DEFAULT_COLORMAP_NORM))
@@ -168,12 +173,37 @@ class ROISelectionWidget(qt.QWidget):
         and show the new data in the stack. Dataset data is not yet replaced.
         A new roi is created in the middle of the new stack.
         """
+        self._abortB.show()
+        self._applyB.setEnabled(False)
+        self._okB.setEnabled(False)
         self.roi = RectangleROI()
         self.roi.setGeometry(origin=self.getRoi().getOrigin(), size=self.getRoi().getSize())
-        self._update_dataset = self.dataset.apply_roi(size=numpy.flip(self.roi.getSize()),
-                                                      center=numpy.flip(self.roi.getCenter()))
-        self.setStack(self._update_dataset)
-        self.resetROI()
+        self.thread = OperationThread(self, self._update_dataset.apply_roi)
+        self.thread.setArgs(size=numpy.flip(self.roi.getSize()),
+                            center=numpy.flip(self.roi.getCenter()))
+        self.thread.finished.connect(self._updateData)
+        self.thread.start()
+
+    def abort(self):
+        self._abortB.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.ROI)
+
+    def _updateData(self):
+        """
+        Updates the stack with the data computed in the thread
+        """
+        self.thread.finished.disconnect(self._updateData)
+        self._abortB.hide()
+        self._abortB.setEnabled(True)
+        self._applyB.setEnabled(True)
+        self._okB.setEnabled(True)
+        if self.thread.data:
+            self._update_dataset = self.thread.data
+            assert self._update_dataset is not None
+            self.setStack(self._update_dataset)
+            self.resetROI()
+        else:
+            print("\nCorrection aborted")
 
     def apply(self):
         """
@@ -183,8 +213,8 @@ class ROISelectionWidget(qt.QWidget):
         """
         if self.roi:
             if self.bg_dataset is not None:
-                self.bg_dataset = self.bg_dataset.apply_roi(self.roi.getSize() / 2,
-                                                            numpy.flip(self.roi.getCenter()))
+                self.bg_dataset = self.bg_dataset.apply_roi(size=numpy.flip(self.roi.getSize()),
+                                                            center=numpy.flip(self.roi.getCenter()))
             self.sigComputed.emit(self.roi.getOrigin().tolist(), self.roi.getSize().tolist())
         else:
             self.sigComputed.emit([], [])
@@ -192,6 +222,23 @@ class ROISelectionWidget(qt.QWidget):
     def getDataset(self):
         return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset
 
+    def getStackViewColormap(self):
+        """
+        Returns the colormap from the stackView
+
+        :rtype: silx.gui.colors.Colormap
+        """
+        return self._sv.getColormap()
+
+    def setStackViewColormap(self, colormap):
+        """
+        Sets the stackView colormap
+
+        :param colormap: Colormap to set
+        :type colormap: silx.gui.colors.Colormap
+        """
+        self._sv.setColormap(colormap)
+
     def resetROI(self):
         """
         Sets the region of interest in the middle of the stack, with size 1/5 of the image.
