@@ -26,12 +26,12 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "08/04/2020"
-
+__date__ = "20/11/2020"
 
+import logging
+_logger = logging.getLogger(__file__)
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Output
-from Orange.widgets import gui
 
 from silx.gui import qt
 
@@ -56,6 +56,7 @@ class DataSelectionWidgetOW(OWWidget):
     filenames = Setting(list(), schema_only=True)
     raw_filename = Setting(str(), schema_only=True)
     dark_filename = Setting(str(), schema_only=True)
+    in_disk = Setting(bool(), schema_only=True)
 
     def __init__(self):
         super().__init__()
@@ -68,8 +69,6 @@ class DataSelectionWidgetOW(OWWidget):
         self.controlArea.layout().addWidget(self._widget)
         self.controlArea.layout().addWidget(_buttons)
 
-        self._progressBar = gui.ProgressBar(self, 100)
-        self._widget.sigProgressChanged.connect(self._updateProgress)
         _buttons.accepted.connect(self._getDataset)
 
         self.__updatingData = False
@@ -79,29 +78,24 @@ class DataSelectionWidgetOW(OWWidget):
         self._widget.setRawFilenames(self.filenames)
         self._widget.setRawFilename(self.raw_filename)
         self._widget.setDarkFilename(self.dark_filename)
+        if self.in_disk:
+            self._widget._inDiskCB.setChecked(True)
 
     def _getDataset(self):
-        assert not self.__updatingData, "Another dataset is being loaded, please wait " \
-                                        "until it has finished"
+        if self.__updatingData:
+            _logger.warning("Another dataset is being loaded, please wait \
+                            until it has finished")
+            return
 
-        print('create new dataset and emit them')
+        _logger.warning('create new dataset and emit them')
         # Create and start thread
-        self._thread = OperationThread(self._widget.loadDataset)
+        self._thread = OperationThread(self, self._widget.loadDataset)
         self._thread.finished.connect(self._sendSignal)
         self._thread.start()
 
         self.updateSettings()
         self.__updatingData = True
         self.information("Downloading dataset")
-        self.close()
-
-    def _updateProgress(self, progress):
-        """
-        Updates the progress bar
-
-        :param int progress: the new value that should appear
-        """
-        self._progressBar.widget.progressBarSet(progress)
 
     def _sendSignal(self):
         """
@@ -112,7 +106,6 @@ class DataSelectionWidgetOW(OWWidget):
         self.__updatingData = False
         self.information()
         self.Outputs.dataset.send(self._widget.getDataset())
-        self._progressBar.finish()
         self.close()
 
     def updateSettings(self):
@@ -122,3 +115,4 @@ class DataSelectionWidgetOW(OWWidget):
         self.filenames = self._widget.getRawFilenames()
         self.raw_filename = self._widget.getRawFilename()
         self.dark_filename = self._widget.getDarkFilename()
+        self.in_disk = self._widget._inDiskCB.isChecked()
