@@ -26,9 +26,11 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "08/04/2020"
+__date__ = "11/12/2020"
 
+from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
+from silx.gui.colors import Colormap
 from darfix.gui.noiseRemovalWidget import NoiseRemovalDialog
 
 
@@ -44,10 +46,18 @@ class NoiseRemovalWidgetOW(OWWidget):
     # Inputs
     class Inputs:
         dataset = Input("dataset", tuple)
+        colormap = Input("colormap", Colormap)
 
     # Outputs
     class Outputs:
         dataset = Output("dataset", tuple)
+        colormap = Output("colormap", Colormap)
+
+    # Settings
+    method = Setting(str())
+    background_type = Setting(str())
+    kernel_size = Setting(str())
+    bottom_threshold = Setting(str())
 
     def __init__(self):
         super().__init__()
@@ -60,16 +70,33 @@ class NoiseRemovalWidgetOW(OWWidget):
     def setDataset(self, dataset):
         self.show()
         if dataset:
-            self._widget.mainWindow.setDataset(*dataset)
+            self._widget.setDataset(*dataset)
+            if self.method:
+                self._widget.mainWindow.method = self.method
+            if self.background_type:
+                self._widget.mainWindow.background = self.background_type
+            if self.kernel_size:
+                self._widget.mainWindow.size = self.kernel_size
+            if self.bottom_threshold:
+                self._widget.mainWindow.bottom_threshold = self.bottom_threshold
         else:
             self._widget.mainWindow.clearStack()
             # Emit None
             self.Outputs.dataset.send(dataset)
 
+    @Inputs.colormap
+    def setColormap(self, colormap):
+        self._widget.mainWindow.setStackViewColormap(colormap)
+
     def _sendSignal(self):
         """
         Function to emit the new dataset.
         """
         self.information()
+        self.method = self._widget.mainWindow.method
+        self.background_type = self._widget.mainWindow.background
+        self.kernel_size = self._widget.mainWindow.size
+        self.bottom_threshold = self._widget.mainWindow.bottom_threshold
         self.Outputs.dataset.send(self._widget.mainWindow.getDataset())
+        self.Outputs.colormap.send(self._widget.mainWindow.getStackViewColormap())
         self.close()
