@@ -26,13 +26,13 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "16/04/2020"
+__date__ = "11/12/2020"
 
 
 from silx.gui.colors import Colormap
 from silx.gui.plot import Plot2D
 from Orange.widgets.widget import OWWidget, Input
-from darfix.gui.utils import ChooseDimensionWidget
+from darfix.gui.utils import ChooseDimensionDock
 
 
 class ZSumWidgetOW(OWWidget):
@@ -48,6 +48,7 @@ class ZSumWidgetOW(OWWidget):
     # Inputs
     class Inputs:
         dataset = Input("dataset", tuple)
+        colormap = Input("colormap", Colormap)
 
     def __init__(self):
         super().__init__()
@@ -55,34 +56,37 @@ class ZSumWidgetOW(OWWidget):
         self._plot = Plot2D(parent=self)
         self._plot.setDefaultColormap(Colormap(name='viridis', normalization='linear'))
         self.controlArea.layout().addWidget(self._plot)
-        self._chooseDimensionWidget = ChooseDimensionWidget(self)
-        self._chooseDimensionWidget.hide()
-        self.controlArea.layout().addWidget(self._chooseDimensionWidget)
-        self._chooseDimensionWidget.filterChanged.connect(self._filterStack)
-        self._chooseDimensionWidget.stateDisabled.connect(self._wholeStack)
+        self._chooseDimensionDock = ChooseDimensionDock(self)
+        self._chooseDimensionDock.hide()
+        self.controlArea.layout().addWidget(self._chooseDimensionDock)
+        self._chooseDimensionDock.widget.filterChanged.connect(self._filterStack)
+        self._chooseDimensionDock.widget.stateDisabled.connect(self._wholeStack)
 
     @Inputs.dataset
     def setDataset(self, dataset):
         # Make sum of dataset data
         if dataset:
             self.dataset = dataset[0]
-            image = self.dataset.zsum()
+            self.indices = dataset[1]
+            if len(self.dataset.data.shape) > 3:
+                self._chooseDimensionDock.show()
+                self._chooseDimensionDock.widget.setDimensions(self.dataset.dims)
+            image = self.dataset.zsum(indices=self.indices)
             self._plot.addImage(image)
             self.open()
-            if len(self.dataset.data.shape) > 3:
-                self._chooseDimensionWidget.show()
-                self._chooseDimensionWidget.setDimensions(self.dataset.dims)
         else:
             self._plot.clear()
 
+    @Inputs.colormap
+    def setColormap(self, colormap):
+        self._plot.getImage().setColormap(colormap)
+
     def _filterStack(self, dim=0, val=0):
-        data = self.dataset.get_data(dimension=[dim, val])
-        if data.shape[0]:
-            image = data.sum(axis=0)
+        image = self.dataset.zsum(indices=self.indices, dimension=[dim, val])
+        if image.shape[0]:
             self._plot.addImage(image)
         else:
             self._plot.clear()
 
     def _wholeStack(self):
-        image = self.dataset.get_data().sum(axis=0)
-        self._plot.addImage(image)
+        self._plot.addImage(self.dataset.zsum())
